/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.eclipse.jdt.annotation.Nullable;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.GenotypeCache;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.PhenotypeCache;

import com.google.common.collect.ClassToInstanceMap;
import com.google.common.collect.MutableClassToInstanceMap;
import com.google.inject.AbstractModule;
import com.google.inject.Scope;

/**
 * Base class for implementing caches using the {@link Scope} mechanism of Guice. Used in the
 * {@link PhenotypeCache} to implement a thread-local cache, and in the {@link GenotypeCache} for a
 * singleton-based variant.
 * 
 * @author diewald
 */
public abstract class EncodingCache<T> extends AbstractModule implements ClassToInstanceMap<T> {

	/** Wrapped map. */
	ClassToInstanceMap<T> map = MutableClassToInstanceMap.create();

	/** {@inheritDoc} */
	@Override
	public int size() {
		return map.size();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEmpty() {
		return map.isEmpty();
	}

	/** {@inheritDoc} */
	@Override
	public boolean containsKey(Object key) {
		return map.containsKey(key);
	}

	/** {@inheritDoc} */
	@Override
	public boolean containsValue(Object value) {
		return map.containsValue(value);
	}

	/** {@inheritDoc} */
	@Override
	public T get(Object key) {
		return map.get(key);
	}

	/** {@inheritDoc} */
	@Override
	public T put(Class<? extends T> key, T value) {
		return map.put(key, value);
	}

	/** {@inheritDoc} */
	@Override
	public T remove(Object key) {
		return map.remove(key);
	}

	/** {@inheritDoc} */
	@Override
	public void putAll(Map<? extends Class<? extends T>, ? extends T> m) {
		map.putAll(m);
	}

	/** {@inheritDoc} */
	@Override
	public void clear() {
		map.clear();
	}

	/** {@inheritDoc} */
	@Override
	public Set<Class<? extends T>> keySet() {
		return map.keySet();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<T> values() {
		return map.values();
	}

	/** {@inheritDoc} */
	@Override
	public Set<java.util.Map.Entry<Class<? extends T>, T>> entrySet() {
		return map.entrySet();
	}

	/** {@inheritDoc} */
	@Override
	public <S extends T> S getInstance(Class<S> type) {
		return map.getInstance(type);
	}

	/** {@inheritDoc} */
	@Override
	public <S extends T> S putInstance(Class<S> type, @Nullable S value) {
		return map.putInstance(type, value);
	}
}
