/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype;

import net.bytebuddy.description.annotation.AnnotationDescription;
import net.bytebuddy.dynamic.DynamicType.Builder.MethodDefinition;
import net.bytebuddy.dynamic.DynamicType.Builder.MethodDefinition.ReceiverTypeDefinition;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingProvider;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingProviderGenerator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.opt4j.core.Genotype;

/**
 * Generator for {@link Genotype} providers. In addition to the {@link EncodingProviderGenerator}, a
 * {@link Genotyped} annoation is added to the Provides method.
 * 
 * @author diewald
 */
public class GenotypeProviderGenerator extends EncodingProviderGenerator<Genotype> {

	/** {@inheritDoc} */
	@Override
	@SuppressWarnings("rawtypes")
	protected MethodDefinition<EncodingProvider> createProvidesMethod(
			ReceiverTypeDefinition<EncodingProvider> withConstructor,
			Class<Genotype> encodingType) {
		MethodDefinition<EncodingProvider> providesMethod =
				super.createProvidesMethod(withConstructor, encodingType);
		return providesMethod
				.annotateMethod(AnnotationDescription.Builder.ofType(Genotyped.class).build());
	}
}
