/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.encoding.IConstrainedEncoding;
import org.fortiss.af3.exploration.model.ExplorationConstraint;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;

/**
 * Base class for {@link Phenotype}s which consists of {@link Phenotype}s.
 * 
 * @author diewald
 */
public abstract class PhenotypeBase implements Phenotype, IConstrainedEncoding {

	/** Flag to mark a phenotype as being processed by sub-Decoders. */
	private boolean isSubDecoding;

	/**
	 * Map holding the constraints on the encoding. Initial capacity is zero to reduce the memory
	 * footprint of non-constrained encodings.
	 */
	private Multimap<Class<? extends ExplorationConstraint<?>>, ExplorationConstraint<?>> constraintMap =
			HashMultimap.create(0, 0);

	/** {@inheritDoc} */
	@Override
	public void setSubDecoding(boolean isSubDecoding) {
		this.isSubDecoding = isSubDecoding;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSubDecoding() {
		return isSubDecoding;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public <T extends ExplorationConstraint<?>> Collection<T> getConstraintsOf(Class<T> type) {
		return (Collection<T>)constraintMap.get(type);
	}

	/** {@inheritDoc} */
	@Override
	public <T extends ExplorationConstraint<?>> void putConstraint(Class<T> type, T constraint) {
		constraintMap.put(type, constraint);
	}

	/** {@inheritDoc} */
	@Override
	public <T extends ExplorationConstraint<?>> void removeConstraints() {
		constraintMap.clear();
	}
}
