/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.encoding;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;
import org.fortiss.af3.exploration.model.ExplorationConstraint;

/**
 * {@link IConstrainedEncoding}s are {@link IExplorationEncoding}s that may carry constraints which
 * limit the allowed operation performed on them.
 * 
 * @author diewald
 */
public interface IConstrainedEncoding {

	/**
	 * Returns the collection of {@link ExplorationConstraint}s that are of the given type.
	 * 
	 * @param type
	 *            of the {@link ExplorationConstraint}s.
	 * @return Collection of {@link ExplorationConstraint}s of the given type, or an empty
	 *         collection if none exist.
	 */
	<T extends ExplorationConstraint<?>> Collection<T> getConstraintsOf(Class<T> type);

	/**
	 * Adds the given {@code constraint} to the set of constraints that is identified by the given
	 * {@code type}.
	 * 
	 * @param type
	 *            Constraint set identifier.
	 * @param constraint
	 *            Constraint to add to possibly existing constraint set.
	 */
	<T extends ExplorationConstraint<?>> void putConstraint(Class<T> type, T constraint);

	/**
	 * Removes the {@link ExplorationConstraint}s from {@code this} encoding. This operation is to
	 * be executed when constraints were invalidated, e.g., due to modifications of depending
	 * encodings.
	 */
	<T extends ExplorationConstraint<?>> void removeConstraints();
}
