/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.taskmapping;

import java.util.Collection;
import java.util.List;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;

import com.google.inject.Inject;

/**
 * Mutate operator that decreases the redundancy of allocations.
 * 
 * @author huang
 */
public class MutateRedundancyDecrement<S extends TaskMappingEntry<ITaskAdapter<?>, IExecutionUnitAdapter<?>>, T extends TaskMappingEncoding<ITaskAdapter<?>, S>>
		extends MutateRedundancy<S, T> {
	/** Constructs a new decrement redundancy mutation operator for a given exploration run */
	@Inject
	public MutateRedundancyDecrement(Collection<ITaskAdapter<?>> tasks) {
		super(tasks);
	}

	/** {@inheritDoc} */
	@Override
	protected void mutate(T encoding, ITaskAdapter<?> deployableComponent, int minReplicatation,
			int maxReplication) {
		List<S> assignedResources = encoding.getMappingEntriesOf(deployableComponent);
		assert (assignedResources != null);

		// Decrease redundancy: keep at least the minimum number of required deployments
		if(assignedResources.size() - 1 >= minReplicatation) {
			assignedResources.remove(rand.nextInt(assignedResources.size()));
		}
	}
}
