/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature;

import static java.util.Arrays.asList;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.safetyfunctionarch.SafetyFunctionArchCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.safetyfunctionarch.SafetyFunctionArchEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.SafeTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.safetyarch.MutateSafetyFunctionArchitectureModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFGraphDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.safetyfunction.SFMappingDecoder;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ISafetyFunctionAdapter;

import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.TypeLiteral;

/**
 * Explores which MooN architecture is suitable for the safety functions present in the input
 * models. This {@link IExplorationFeature} affects both the task model as well as the logical
 * architecture since the behaviour of the system could be modified.
 * 
 * @author diewald
 */
public class SafetyArchitectureExploration extends ExplorationFeature {

	/** Constructor. */
	public SafetyArchitectureExploration() {
		super("Safety Architecture Exploration");

		// Variables
		addVariable(SafetyFunctionArchEncoding.class, new SafetyFunctionArchCreator(),
				asList(new MutateSafetyFunctionArchitectureModule()));
		// Decoding
		addProblem(AbstractTaskMappingEncoding.class, new SFMappingDecoder());
		// TODO: ConstraintDecoder
		// getService(IExplorationModuleService.class).registerProblem(
		// AbstractTaskMappingEncoding.class, new SFMappingConstraintDecoder());
		addProblem(SafeTaskGraphEncoding.class, new SFGraphDecoder());
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		Collection<ISafetyFunctionAdapter<?>> sfAdps = dseInjector
				.getInstance(Key.get(new TypeLiteral<Collection<ISafetyFunctionAdapter<?>>>() {/**/
				}));
		return sfAdps.isEmpty() ? false : true;
	}
}
